<?php
// $Id: profile_setup.page_account_category.inc,v 1.1.2.1 2009/03/24 07:28:55 thomjjames Exp $

/**
 * @file
 * Page user edit type
 */

include_once(drupal_get_path('module', 'pageroute') .'/pageroute.page.inc');

class profile_setup_page_account_category extends pageroute_page {

  public function set_form(&$form, &$form_state, &$args) {
    $form = array();
    global $user;
    $account = user_load(array('uid' => $user->uid));

    if (!$account->uid) {
      drupal_not_found();
      pageroute_exit_now();
    }

	  $route = $form_state['storage']['route'];
    $args['hide_pageroute_buttons'] = FALSE;
    $args['default_target'] = PAGEROUTE_FORWARD;

    $skip_button = array(
	    '#type' => 'submit',
      '#value' => t('Skip'),
      '#weight' => 100
    );
	  //show back button if not first step
	  if ($route->page_index[$route->page_data_current->name] > 0 && !$this->options['hide_back_button']) {
      $back_button = array(
	    '#type' => 'submit',
        '#value' => t('Back'),
        '#weight' => 20
      );	
	  }
    	
	  //profile and content profile forms need rendering differently
    if (profile_setup_page_is_content_profile_type($this->options['category'])) {
      $type = $this->options['category'];
	    $form_id = $type .'_node_form';
	    $node = content_profile_load($type, $account->uid);
      if (!$node) {
        $node = array('uid' => $account->uid, 'name' => (isset($account->name) ? $account->name : ''), 'type' => $type, 'language' => '');
      }
	  
	    $form_state['values'] = $_POST; //recreate values as pageroute.module unsets them
      $form += drupal_retrieve_form($form_id, $form_state, $node);
      drupal_prepare_form($form_id, $form, $form_state);
	    $form_type = 'content_profile';
	    unset($form['buttons']['preview']);
	    unset($form['buttons']['delete']);
	    $form['buttons']['skip'] = $skip_button;	
      $form['buttons']['back'] = $back_button;
      $form['buttons']['submit']['#weight'] = 50;	  
    }
	  else {
      $form += drupal_retrieve_form('user_profile_form', &$form_state, $account, $this->options['category']);
	    $form_type = 'user_profile';
      drupal_prepare_form('user_profile_form', &$form, &$form_state);
	    unset($form['delete']);
	    $form['skip'] = $skip_button;
      $form['back'] = $back_button;	  
    }
    $form['description'] = array(
      '#value' => $this->options['description'],
      '#weight' => -100
    );	
	  $form['form_type'] = array(
	    '#value' => $form_type,
	    '#type' => 'value'
    );
    return $form;
  }

  public static function ui($page, &$form) {
    $categories = array();
    foreach (_user_categories(false) as $category) {
      $categories[$category['name']] = $category['title'];
    }
    $form['options']['category'] = array(
      '#type' => 'select',
      '#title' => t('Category'),
      '#description' => t('The form of the chosen category will be used for this page.'),
      '#required' => TRUE,
      '#default_value' => isset($page->options['category']) ? $page->options['category'] : '',
      '#weight' => 2,
      '#options' => $categories,
    );
    $form['options']['description'] = array(
	    '#type' => 'textarea',
      '#title' => t('Form description'),
      '#default_value' => $page->options['description'],
      '#description' => t('You can optionally input a description or message that will appear at the top of this form.')
    );
    $form['options']['hide_back_button'] = array(
	    '#type' => 'checkbox',
      '#title' => t('Hide back button?'),
      '#default_value' => $page->options['hide_back_button'],
      '#description' => t('Check this box to hide the form "back" button.')
    );	
		$options = array('forward', 'back', 'cancel');
		foreach ($options as $button) {
      $form['options'][$button]['#prefix'] = '<div style="display: none;">';
			$form['options'][$button]['#suffix'] = '</div>';
		}
  }

  public static function help() {
    $help[] = t('This page type is provided by the profile_setup module to allow users to edit their profile "edit" categories which can be created using the profile or content profile modules.');
    $help[] = t('<strong>Using Content Profile Module:</strong> If you are using Content Profile you much select <em>Show a secondary tab below the user\'s edit tab</em> under <em>Profile edit tab</em> in the content profile content type tab per content type.');
    $help[] = t('<strong>Using Profile Module:</strong> If you are using core Profile account categories are automatically created as you create profile fields.');
    return implode('<br />', $help);
  }

  public static function info() {
    return array('name' => t('User account editing form'));
  }

  public static function form_submitted(&$form_state) {

  $todo = NULL;
  $args = &$form_state['storage']['route']->args;
  if ($form_state['clicked_button']['#value'] == t('Save') || $form_state['clicked_button']['#value'] == t('Skip')) {
    $args['default_target'] = PAGEROUTE_FORWARD;
	  switch ($form_state['values']['form_type']) {
      case 'user_profile':
        if (profile_setup_user_profile_form_validate($form_state)) {
          if ($form_state['clicked_button']['#value'] == t('Save')) {
		        profile_setup_user_profile_form_submit($form_state);
		      }
        }
        else {
		      //if validation errors return to current page
		      $args['default_target'] = PAGEROUTE_CURRENT;
        }
        break;
      case 'content_profile':
	    //do nothing - allow node form validation
      break;
    }
  }
  
  //back buttons
  if ($form_state['clicked_button']['#value'] == t('Back')) {
    $args['default_target'] = PAGEROUTE_BACK;
  }  
	

  return $todo;
  }

  protected function set_up() {
	  include_once(drupal_get_path('module', 'node') .'/node.pages.inc');
		include_once(drupal_get_path('module', 'user') .'/user.pages.inc');
	}
}

function profile_setup_page_is_content_profile_type($category) {
  return is_content_profile($category);
}

/**
 * user_profile type form validation
 * 
 * Because _user_edit_validate() function passes arg(1) 
 * from user_user() we can just use user_module_invoke as our URL may not contain arg(1)
 */
function profile_setup_user_profile_form_validate(&$form_state) {
  profile_setup_user_invoke('validate', $form_state['values'], $form_state['values']['_account'], $form_state['values']['_category'], array('user'));
  // Validate input to ensure that non-privileged users can't alter protected data.
  if ((!user_access('administer users') && array_intersect(array_keys($form_state['values']), array('uid', 'init', 'session'))) || (!user_access('administer permissions') && isset($form_state['values']['roles']))) {
    watchdog('security', 'Detected malicious attempt to alter protected user fields.', array(), WATCHDOG_WARNING);
    // set this to a value type field
    form_set_error('category', t('Detected malicious attempt to alter protected user fields.'));
    return FALSE;
  }
  if ($errors = form_get_errors()) {
    return FALSE;
  }
  return TRUE;
}

/**
 * user_profile type form submission
 */
function profile_setup_user_profile_form_submit(&$form_state) {
  $account = $form_state['values']['_account'];
  $category = $form_state['values']['_category'];
  unset($form_state['values']['_account'], $form_state['values']['op'], $form_state['values']['submit'], $form_state['values']['delete'], $form_state['values']['form_token'], $form_state['values']['form_id'], $form_state['values']['_category']);
  user_module_invoke('submit', $form_state['values'], $account, $category);
  user_save($account, $form_state['values'], $category);

  // Clear the page cache because pages can contain usernames and/or profile information:
  cache_clear_all();

  drupal_set_message(t('The changes have been saved.'));
  return;
}

/**
 * custom profile validation function
 */
function profile_setup_user_invoke($type, &$array, &$user, $category = NULL, $exclude = array()) {
  if ($category == 'account') {
    _user_edit_validate($user->uid, $array); //invoke user module separately
  }
  foreach (module_list() as $module) {
    if (!in_array($module, $exclude)) {
	  $function = $module .'_user';
      if (function_exists($function)) {
        $function($type, $array, $user, $category);
      }
	}
  }
}
