<?php
// $Id: profile_setup.page_profile_picture.inc,v 1.1.2.1 2009/03/24 07:28:55 thomjjames Exp $

/**
 * @file
 * Page user edit type
 */

include_once(drupal_get_path('module', 'pageroute') .'/pageroute.page.inc');

class profile_setup_page_profile_picture extends pageroute_page {  

  public function set_form(&$form, &$form_state, &$args) {
    
    $args['hide_pageroute_buttons'] = FALSE;
    $args['default_target'] = PAGEROUTE_FORWARD;

    $form += drupal_retrieve_form('profile_setup_profile_picture_form', $form_state);
    //manipulate buttons
    $form['bottom']['buttons']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
      '#weight' => 50,
      '#prefix' => ''
    );
    $form['bottom']['buttons']['back']['#weight'] = 20;
    $form['bottom']['buttons']['forward']['#weight'] = 100;		
    $form['description'] = array(
	    '#value' => $this->options['description'],
      '#weight' => -100
    );		
    return;
  }

  public static function ui($page, &$form) {			
		$form['options']['description'] = array(
		  '#type' => 'textarea',
			'#title' => t('Form description'),
			'#default_value' => $page->options['description'],
			'#description' => t('You can optionally input a description or message for this form to help encourage users to fill it out.')
		);
  }

  public static function help() {
    $help[] = t('This page type is provided by the profile_setup module to allow you to create a profile setup workflow page where users can upload a profile picture.'); 
		$help[] = t('NOTE: Creating this page type will overide your picture support settings on the !user_admin page.', array('!user_admin' => l(t('user settings'), 'admin/user/settings')));
		return implode('<br />', $help);  
  }
  
  public static function info() {
    return array('name' => t('User profile picture upload form'));
  }
  
  public static function form_submitted(&$form_state) {
    $todo = NULL;
    $args = &$form_state['storage']['route']->args;

    if ($form_state['clicked_button']['#value'] == t('Save')) {//use class not value as value can be changed
      $args['default_target'] = PAGEROUTE_FORWARD;
	    if (profile_setup_picture_form_submit($form, &$form_state)) {    		
	      if (is_numeric($form_state['clicked_button']['#name'])) {
          $target = $form_state['clicked_button']['#name'];
        }
        else {
          $target = $args['todo']['target'];
        }
        $todo = array('action' => $form_state['clicked_button']['#value'], 'target' => $target);						
      }
      else {
        $args['default_target'] = PAGEROUTE_CURRENT;	  
      }
    }    
    return $todo;
  }
  
  public function set_up() {
    //nothing to add here
  }
}

/**
 * default step 1 - upload a profile picture
 */
function profile_setup_profile_picture_form() {
  global $user;
  $edit = $user;
  $form['picture'] = array(
    '#type' => 'fieldset',
    '#title' => t('Your Profile Picture'),
    '#weight' => 1,
  );
  $form['picture']['picture_delete'] = array('#type' => 'hidden');
  $form['picture']['picture_upload'] = array(
    '#type' => 'file',
    '#title' => t('Upload a profile picture'),
    '#size' => 48,
    '#description' => t('Your virtual face or picture. Maximum dimensions are %dimensions and the maximum size is %size kB.', array('%dimensions' => variable_get('user_picture_dimensions', '85x85'), '%size' => variable_get('user_picture_file_size', '30'))) .' '. variable_get('user_picture_guidelines', '')
  );

  $form['#attributes'] = array('enctype' => 'multipart/form-data');
	
  return $form;
}

/**
 * validation
 */
function _profile_setup_profile_picture_validate($file) {
  global $user;

  // Check that uploaded file is an image, with a maximum file size
  // and maximum height/width.
  $info = image_get_info($file->filepath);
  list($maxwidth, $maxheight) = explode('x', variable_get('user_picture_dimensions', '85x85'));

  if (!$info || !$info['extension']) {
    form_set_error('picture_upload', t('The uploaded file was not an image.'));
    return FALSE;
  }
  else if (image_get_toolkit()) {
    image_scale($file->filepath, $file->filepath, $maxwidth, $maxheight);
  }
  else if (filesize($file->filepath) > (variable_get('user_picture_file_size', '30') * 1000)) {
    form_set_error('picture_upload', t('The uploaded image is too large; the maximum file size is %size kB.', array('%size' => variable_get('user_picture_file_size', '30'))));
    return FALSE;
  }
  else if ($info['width'] > $maxwidth || $info['height'] > $maxheight) {
    form_set_error('picture_upload', t('The uploaded image is too large; the maximum dimensions are %dimensions pixels.', array('%dimensions' => variable_get('user_picture_dimensions', '85x85'))));
    return FALSE;
  }

  //Rename picture with unique id - take from unique_avatar.module
  $dest = variable_get('user_picture_path', 'pictures') .'/picture-'. $user->uid .'-'. md5(time()) .'.'. $info['extension'];
  if (file_move($file->filepath, $dest)) {

    //Clean up existing picture
    if (module_exists('imagecache')) {
      imagecache_image_flush($user->picture);
    }
    file_delete($user->picture);

    //Assign new picture
    $edit['picture'] = file_directory_path() .'/'. $dest;
    $user->picture = file_directory_path() .'/'. $dest;
    db_query("UPDATE {users} SET picture = '%s' WHERE uid = %d", $user->picture, $user->uid);
    return TRUE;
  }
  return FALSE;
}

/**
 * validate and store user profile picture then redirect to next step
 */
function profile_setup_picture_form_submit($form, &$form_state) {
  global $user;
    // If required, validate the uploaded picture.
    $validators = array();

    if ($file = file_save_upload('picture_upload', $validators)) {
			if (!_profile_setup_profile_picture_validate($file)) {
			  return false;
			}
			else {
        drupal_set_message(t('Your profile picture was successfully uploaded'));
				return true;
      }
    }
		else {
      return true;
	  }		
}
