<?php
// $Id: profile_setup.page_username_change.inc,v 1.1.2.1 2009/03/24 07:28:55 thomjjames Exp $

/**
 * @file
 * Page user edit type
 */

include_once(drupal_get_path('module', 'pageroute') .'/pageroute.page.inc');

class profile_setup_page_username_change extends pageroute_page {  

  public function set_form(&$form, &$form_state, &$args) {
    if (!user_access('change own username')) {
      drupal_access_denied();
      pageroute_exit_now();    
    }
    $args['hide_pageroute_buttons'] = FALSE;
    $args['default_target'] = PAGEROUTE_FORWARD;

    //add our save button
    $form['bottom']['buttons']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
      '#weight' => -20,
      '#prefix' => ''
    );
    
    $form += drupal_retrieve_form('profile_setup_username_form', $form_state);
    $form['description'] = array(
      '#value' => $this->options['description'],
      '#weight' => -100
    );		
    return;
  }

  public static function ui($page, &$form) {
		$form['options']['description'] = array(
		  '#type' => 'textarea',
			'#title' => t('Form description'),
			'#default_value' => $page->options['description'],
			'#description' => t('You can optionally input a description or message for this form to help encourage users to fill it out.')
		);
  }

  public static function help() {
    $help[] = t('This page type is provided by the profile_setup module to allow users to change their username.'); 
		$help[] = t('NOTE: You will need to enable the "change own username" permission to allow users to use this form. !user_admin.', array('!user_admin' => l(t('View current permissions'), 'admin/user/permissions')));
		return implode('<br />', $help);  
  }
  
  public static function info() {
    return array('name' => t('Change username form'));
  }
  
  public static function form_submitted(&$form_state) {
    $todo = NULL;
    $args = &$form_state['storage']['route']->args;

    if ($form_state['clicked_button']['#value'] == t('Save')) {
      $args['default_target'] = PAGEROUTE_FORWARD;
			if (!profile_setup_username_form_validate($form, $form_state)) {
			  //validation failed return to same page	
				$args['default_target'] = PAGEROUTE_CURRENT;
			}
			else {			
			  if (profile_setup_username_form_submit($form, $form_state)) {   
				  if (is_numeric($form_state['clicked_button']['#name'])) {
            $target = $form_state['clicked_button']['#name'];
          }
          else {
            $target = $args['todo']['target'];
          }
          $todo = array('action' => $form_state['clicked_button']['#value'], 'target' => $target);
			  }
			}
    }    
    return $todo;
  }
  
  public function set_up() {
    //nothing to add here
  }
}


/**
 * change username form
 */
function profile_setup_username_form(&$form_state) {
  global $user;
  $form['account']['name'] = array('#type' => 'textfield',
    '#title' => t('Username'),
    '#default_value' => $user->name,
    '#maxlength' => USERNAME_MAX_LENGTH,
    '#description' => t('Your preferred username; punctuation is not allowed except for periods, hyphens, and underscores.'),
    '#required' => TRUE,
  );
  $form['_account'] = array(
    '#type' => 'value',
    '#value' => $user,
  );
  return $form;
}

/**
 * validate and save username
 */
function profile_setup_username_form_validate($form, &$form_state) {
  global $user;
  if ($form_state['values']['name']) {
    // only validate if they've changed the username
    if ($form_state['values']['name'] != $user->name) {
      if ($error = user_validate_name($form_state['values']['name'])) {
        form_set_error('name', $error);
				return false;
      }
      else if (db_result(db_query("SELECT COUNT(*) FROM {users} WHERE uid <> %d AND LOWER(name) = LOWER('%s')", $uid, $form_state['values']['name'])) > 0) {
        form_set_error('name', t('The name %name is already taken.', array('%name' => $form_state['values']['name'])));
				return false;
      }
      else if (drupal_is_denied('user', $form_state['values']['name'])) {
        form_set_error('name', t('The name %name has been denied access.', array('%name' => $form_state['values']['name'])));
				return false;
      }
    }
  }
	return true;
}

/**
 * validate and save username
 */
function profile_setup_username_form_submit($form, &$form_state) {
  global $user;
  if ($form_state['values']['name']) {
    if ($form_state['values']['name'] != $user->name) {
      $account = $form_state['values']['_account'];
      unset($form_state['values']['_account'], $form_state['values']['op'], $form_state['values']['submit'], $form_state['values']['delete'], $form_state['values']['form_token'], $form_state['values']['form_id'], $form_state['values']['_category']);
      if ($new = user_save($account, $form_state['values'])) {
        $user->name = $new->name;
				drupal_set_message(t('Your username was successfully updated'));
				return true;
      }
    }
  }
}
